<template>
  <button
    ref="element"
    class="btn btn-white p-0"
    type="button"
    @click.prevent="toggle"
  >
    <slot name="button">
      <EllipsisRegularIcon class="m-1.5 size-4" />
    </slot>
  </button>
  <div
    v-if="display"
    ref="floating"
    :style="floatingStyles"
    class="absolute z-10 flex w-56 flex-col rounded border border-gray-200 bg-white py-1 text-sm font-normal drop-shadow-xl"
  >
    <slot v-bind="{ close, open, toggle }" />
  </div>
</template>

<script lang="ts" setup>
import { offset, shift, flip, useFloating } from "@floating-ui/vue";
import { onClickOutside, onKeyDown, useElementVisibility, useScroll } from "@vueuse/core";
import EllipsisRegularIcon from "@/svg/icons/ellipsis-regular.svg?component";
import type { Ref } from "vue";

type Alignment = "start" | "end";
type Side = "top" | "right" | "bottom" | "left";
type AlignedPlacement = `${Side}-${Alignment}`;
type TooltipProvide = {
  open: () => void;
  close: () => void;
  toggle: () => void;
};

const props = defineProps<{
  placement?: AlignedPlacement;
}>();

const reference = inject<Ref<HTMLElement>>("reference");

const display = ref(false);
const element = ref<HTMLElement>();
const floating = ref<HTMLElement>();
const { floatingStyles } = useFloating(element, floating, { placement: props.placement, middleware: [shift(), offset(8), flip()] });

onKeyDown("Escape", (e) => {
  if (display.value) {
    close();
    e.preventDefault();
  }
});
const isVisible = useElementVisibility(element);
onClickOutside(floating, () => close(), { ignore: [element] });
useScroll(reference?.value ?? null, { onScroll: () => (display.value ? close() : void 0) });

const open = () => {
  if (isVisible.value) {
    display.value = true;
  }
};
const close = () => {
  display.value = false;
};
const toggle = () => {
  !display.value ? open() : close();
};

provide<TooltipProvide>("tooltip", { open: open, close: close, toggle: toggle });
</script>
